package oracle.forms.fd;

import java.awt.*;
import java.applet.*;
import java.awt.event.*;
import java.awt.geom.RoundRectangle2D;
import java.util.StringTokenizer;

  /*
   * Class to display/hide the tooltip
   */
   
public class MLToolTip extends Canvas {

  private   oracle.forms.ui.ExtendedFrame ef ;
	protected String        tip;
  protected String []     ttips = new String[30] ;
	protected Component     owner;
	private   Container     mainContainer;
	private   LayoutManager mainLayout;
  private   String sSep = "|" ;	
  private   int    iCWidth=0, iCHeight=0 ;
  private   int    iTWidth=0, iTHeight=0 ;
  private   int    iWinPosX=0, iWinPosY=0 ;
	private   boolean shown;
  private   boolean bAttachToMDI = false ; 	
  private   boolean bLog = false ;
	private final int VERTICAL_OFFSET = 5;
	private final int HORIZONTAL_ENLARGE = 25;
  private int       iX=0, iY=0;
  private int       iLines = -1 ;
  private int       iLineHeight = 0 ;
  private int       iMaxWidth  = 0 ;
  private int       iMaxHeight = 0 ;
  private Color     cStart = new Color(255,255,220) ;
  private Color     cEnd   = new Color(255,255,220) ;
  private Color     cText  = Color.black ;
	
  public MLToolTip(String tip, Component owner) {
     getWindow(owner);
     this.tip   = tip;
     this.owner = owner;
     getParams(tip);
     setLines(this.tip) ;
     owner.addMouseListener(new MAdapter());
     setBackground(new Color(255,255,220));
   }
   
  
  protected void setColors(Color cStart, Color cEnd)
  {
    this.cStart = cStart ;
    this.cEnd   = cEnd ;    
  }
  
  protected void setTextColor(Color c)
  {
    this.cText = c ;
  }  

  protected void setPosition(int x, int y)
  {
    this.iX = x ;
    this.iY = y ;
  }  
  
  protected void setAttach(boolean b)
  {
    bAttachToMDI = b ;
  }
  
	protected void setLog(boolean b)
  {
    bLog = b ;  
  }
  
  private void setLines(String s)
  {
    iLines = -1 ;
    log("setLines:"+s);
    StringTokenizer st = new StringTokenizer(s,sSep);
    if(! st.hasMoreTokens())
    {
      iLines++;
      ttips[iLines] = s; 
    }
    else
    {
      while(st.hasMoreTokens())
      {
        iLines++;
        ttips[iLines] = st.nextToken() ;
      }
    }
    iMaxHeight = 0 ;
    iMaxWidth  = 0 ;
    FontMetrics fm = getFontMetrics(this.owner.getFont());
    iLineHeight = fm.getHeight() ;
    for(int i=0 ; i<=iLines; i++)
    {
      //log("line=:"+ttips[i]);
      iMaxHeight += iLineHeight ;
      iMaxWidth = (fm.stringWidth(ttips[i]) > iMaxWidth ? fm.stringWidth(ttips[i]) : iMaxWidth );
    }
  }
  
  private void getParams(String s)
  {
    int iStart = s.indexOf("<params>") ;
    int iStop = s.indexOf("</params>") ;
    if((iStart > -1) && (iStop > -1))
    {
      String sParams = s.substring(iStart+8,iStop) ;
      log("getParams() : "+sParams);
      this.tip = s.substring(0,iStart) + s.substring(iStop+9) ;
      // foreground color ?
      iStart = sParams.indexOf("fg=") ;
      iStop  = sParams.indexOf(" ",iStart) ;
      if(iStop == -1) iStop  = sParams.length() ;
      if((iStart > -1) && (iStop > -1))
      {
        String sColor = sParams.substring(iStart+3,iStop);
        log("color="+sColor);
        int r=-1, g=-1, b=-1, iNum = 0 ;
        StringTokenizer st = new StringTokenizer(sColor,",");
        try{
          r = Integer.parseInt(st.nextToken()) ;
          g = Integer.parseInt(st.nextToken()) ;
          b = Integer.parseInt(st.nextToken()) ;    
          setTextColor(new Color(r,g,b));
        }
        catch(Exception e) {e.printStackTrace();}        
      }
      // background color ?
      iStart = sParams.indexOf("bg=") ;
      iStop  = sParams.indexOf(" ",iStart) ;
      log("iStart="+iStart+"  iStop="+iStop);
      if(iStop == -1) iStop  = sParams.length() ;
      if((iStart > -1) && (iStop > -1))
      {
        String sColor = sParams.substring(iStart+3,iStop);
        log("colors="+sColor);
        Color c1=null, c2=null ;
        int r=-1, g=-1, b=-1;
        StringTokenizer st = new StringTokenizer(sColor,",");
        try{
          r = Integer.parseInt(st.nextToken()) ;
          g = Integer.parseInt(st.nextToken()) ;
          b = Integer.parseInt(st.nextToken()) ;    
          c1 = new Color(r,g,b) ;
          setColors(c1,c1);
        }
        catch(Exception e) {e.printStackTrace();}
        if(st.hasMoreTokens())
        {
          try{        
            r=-1; g=-1; b=-1;
            r = Integer.parseInt(st.nextToken()) ;
            g = Integer.parseInt(st.nextToken()) ;
            b = Integer.parseInt(st.nextToken()) ;    
            c2 = new Color(r,g,b) ;         
            setColors(c1,c2);
          }
          catch(Exception e) {e.printStackTrace();}
        }
      }      
      // Position ?
      iStart = sParams.indexOf("pos=") ;
      iStop  = sParams.indexOf(" ",iStart) ;
      if(iStop == -1) iStop  = sParams.length() ;
      if((iStart > -1) && (iStop > -1))
      {
        String st = sParams.substring(iStart+4,iStop);
        log("pos="+st);
        int x=0, y=0 ;
        StringTokenizer stk = new StringTokenizer(st,",");
        try{
          x = Integer.parseInt(stk.nextToken()) ;
          y = Integer.parseInt(stk.nextToken()) ;
          log("pos x="+x+"  y="+y);
          setPosition(x,y);
        }
        catch(Exception e) {e.printStackTrace();}        
      }
    }
  }

  /*
   *  get the parent window of the component
   */
  private void getWindow(Component parent)
  {
    log("getWindow()");
    boolean bCont  = true ;
    Container cont = parent.getParent();
    while (cont!=null)
    {
      // Search for the Extended Frame
      if(cont.getName().startsWith("ExtendedFrame"))
      {
        //  current window  
        ef = (oracle.forms.ui.ExtendedFrame)cont ;
        return ;
      }
      cont = cont.getParent() ;
    }
  }  
  
  /*
   * draw the tooltip
   */
  public void paint(Graphics g) {
	  Graphics2D g2 = (Graphics2D) g ;
	  // shadow rectangle
    g2.setColor(new Color(180,180,180));
	  g2.fill(new RoundRectangle2D.Double(3, 3,
	                                     iTWidth+6,
	                                     iTHeight,
	                                     10, 10));        
    if(cStart != cEnd)
    {
	    GradientPaint redtowhite = new GradientPaint(0,0,cStart,iTWidth+6,iTHeight,cEnd);
	    g2.setPaint(redtowhite);
	  }
    else
      g2.setPaint(cStart);
    // tooltip background
    g2.fill(new RoundRectangle2D.Double(0, 0,
	                                     iTWidth+6,
	                                     iTHeight,
	                                     10, 10));    
		// tooltip border
    g2.setColor(Color.black);
	  g2.draw(new RoundRectangle2D.Double(0, 0,
	                                     iTWidth+6,
	                                     iTHeight,
	                                     10, 10));    
	  // tooltip text
    g2.setColor(cText);
    int iHight = iTHeight - (iLines * iLineHeight);
    for(int i=0 ; i<=iLines; i++)
    {
      g2.drawString(ttips[i], 5, iHight - 6);
      iHight += iLineHeight ;
    }
	}

	private void addToolTip() {
		mainContainer.setLayout(null);
		
		FontMetrics fm = getFontMetrics(owner.getFont());    		
	  
	  iTWidth  = iMaxWidth + HORIZONTAL_ENLARGE ;
    iTHeight = iMaxHeight + 5 ;
	  iCWidth  = iTWidth + 10 ;
	  iCHeight = iTHeight + 10 ; 
    setSize(iCWidth, iCHeight);

    if(iX==0 && iY==0)
    {
      // tooltip location relative to its parent item
      setLocation((owner.getLocationOnScreen().x - mainContainer.getLocationOnScreen().x) , 
                  (owner.getLocationOnScreen().y - mainContainer.getLocationOnScreen().y 
                  + VERTICAL_OFFSET
                  + owner.getHeight())) ;
    }
    else
    {
      // tooltip absolute location
      if(! bAttachToMDI)
      {
        // tooltip attached to the parent window (not MDI)
        // get the parent window current coordinates
        iWinPosX = (int)ef.getLocation().getX() ;
        iWinPosY = (int)ef.getLocation().getY() ;
      }
      setLocation(iWinPosX+iX, iWinPosY+iY);
    }

		// correction, whole tool tip must be visible 
		if (mainContainer.getSize().width < ( getLocation().x + getSize().width )) {
			setLocation(mainContainer.getSize().width - getSize().width, getLocation().y);
		}
		mainContainer.add(this, 0);
		mainContainer.validate();
		repaint();
		shown = true;
	}

	
	private void removeToolTip() {
    if (shown) {
      mainContainer.remove(0);
			mainContainer.validate();
		}
		shown = false;
	}

	private void findMainContainer() {
		Container parent = owner.getParent();
		while (true) {
			if ((parent instanceof Applet) || (parent instanceof Frame)) {
				mainContainer = parent;
				break;				
			} else {
				parent = parent.getParent();
			}
		}		
		mainLayout = mainContainer.getLayout();
	}

  private void log(String sMessage)
  {
    if(bLog) System.out.println(sMessage);
  }
  
    class MAdapter extends MouseAdapter {
    	public void mouseEntered(MouseEvent me) {
		    findMainContainer();
    		addToolTip();
    	}
	    public void mouseExited(MouseEvent me) {
	    	removeToolTip();
		}
		public void mousePressed(MouseEvent me) {
	    	removeToolTip();
		}
	}
}
